@@ -5,6 +5,7 @@ import pandas as pd
 from .transformer import is_lambda
 import types
 import requests
+from .io import autoopen
 from . import tqdm, HOME_DIR
 import tarfile
 
@@ -32,12 +33,24 @@ class Dataset():
     def _configure(self, **kwargs):
         pass
 
-    def get_corpus_location(self):
+    def get_corpus(self):
         """ 
             Returns the location of the files to allow indexing the corpus, i.e. it returns a list of filenames.
         """
         pass
 
+    def get_corpus_iter(self, verbose=True):
+        """
+            Returns an iter of dicts for this collection. If verbose=True, a tqdm pbar shows the progress over this iterator.
+        """
+        pass
+
+    def get_corpus_lang(self):
+        """
+            Returns the ISO 639-1 language code for the corpus, or None for multiple/other/unknown
+        """
+        return None
+
     def get_index(self, variant=None):
         """ 
             Returns the IndexRef of the index to allow retrieval. Only a few datasets provide indices ready made.
@@ -50,12 +63,24 @@ class Dataset():
         """
         pass
 
+    def get_topics_lang(self):
+        """
+            Returns the ISO 639-1 language code for the topics, or None for multiple/other/unknown
+        """
+        return None
+
     def get_qrels(self, variant=None):
         """ 
             Returns the qrels, as a dataframe, ready for evaluation.
         """
         pass
 
+    def info_url(self):
+        """
+            Returns a url that provides more information about this dataset.
+        """
+        return None
+
 class RemoteDataset(Dataset):
 
     def __init__(self, name, locations):
@@ -150,7 +175,7 @@ class RemoteDataset(Dataset):
                 raise ValueError("Could not fetch " + URL) from he
         return (local, filetype)
 
-    def _get_all_files(self, component, **kwargs):
+    def _get_all_files(self, component, variant=None, **kwargs):
         localDir = os.path.join(self.corpus_home, component)
         if not os.path.exists(localDir):
             os.makedirs(localDir)
@@ -158,7 +183,8 @@ class RemoteDataset(Dataset):
         kwargs = {}
         if self.user is not None:
             kwargs["auth"]=(self.user, self.password)
-        for (local, URL) in self.locations[component]:
+        file_list = self.locations[component] if variant is None else self.locations[component][variant]
+        for (local, URL) in file_list:
             local = os.path.join(localDir, local)
             if not os.path.exists(local):
                 if "#" in URL:
@@ -189,6 +215,16 @@ class RemoteDataset(Dataset):
         import pyterrier as pt
         return pt.io.find_files(self._get_all_files("corpus", **kwargs))
 
+    def get_corpus_iter(self, **kwargs):
+        if not "corpus_iter" in self.locations:
+            raise ValueError("Cannot supply a corpus iterator on dataset %s" % self.name)
+        return self.locations["corpus_iter"](self, **kwargs)
+        
+    def get_corpus_lang(self):
+        if 'corpus' in self.locations:
+            return 'en' # all are english
+        return None
+
     def get_qrels(self, variant=None):
         import pyterrier as pt
         filename, type = self._get_one_file("qrels", variant)
@@ -204,15 +240,156 @@ class RemoteDataset(Dataset):
         elif filetype == "direct":
             return file
         raise ValueError("Unknown filetype %s for %s topics %s"  % (filetype, self.name, variant))
+    
+    def get_topics_lang(self):
+        if 'topics' in self.locations:
+            return 'en' # all are english
+        return None
 
-    def get_index(self):
+    def get_index(self, variant=None):
         import pyterrier as pt
-        thedir = self._get_all_files("index")
+        if self.name == "50pct" and variant is None:
+            variant="ex1"
+        thedir = self._get_all_files("index", variant=variant)
         return pt.autoclass("org.terrier.querying.IndexRef").of(os.path.join(thedir, "data.properties"))
 
     def __repr__(self):
         return "RemoteDataset for %s, with %s" % (self.name, str(list(self.locations.keys())))
 
+    def info_url(self):
+        return self.locations['info_url'] if "info_url" in self.locations else None
+
+
+class IRDSDataset(Dataset):
+    def __init__(self, irds_id):
+        self._irds_id = irds_id
+        self._irds_ref = None
+
+    def irds_ref(self):
+        if self._irds_ref is None:
+            self._irds_ref = ir_datasets.load(self._irds_id)
+        return self._irds_ref
+
+    def get_corpus(self):
+        raise NotImplementedError("IRDSDataset doesn't support get_corpus; use get_corpus_iter instead")
+
+    def get_corpus_iter(self, verbose=True):
+        ds = self.irds_ref()
+        assert ds.has_docs(), f"{self._irds_id} doesn't support get_corpus_iter"
+        it = ds.docs_iter()
+        if verbose:
+            it = tqdm(it, desc=f'{self._irds_id} documents', total=ds.docs_count())
+        for doc in it:
+            doc = doc._asdict()
+            # pyterrier uses "docno"
+            doc['docno'] = doc.pop('doc_id')
+            yield doc
+
+    def get_corpus_lang(self):
+        ds = self.irds_ref()
+        if ds.has_docs():
+            return ds.docs_lang()
+        return None
+
+    def get_index(self, variant=None):
+        # this is only for indices where Terrier provides an index already
+        raise NotImplementedError("IRDSDataset doesn't support get_index")
+
+    def get_topics(self, variant=None):
+        """
+            Returns the topics, as a dataframe, ready for retrieval. 
+        """
+        ds = self.irds_ref()
+        assert ds.has_queries(), f"{self._irds_id} doesn't support get_topics"
+        qcls = ds.queries_cls()
+        assert variant is None or variant in qcls._fields[1:], f"{self._irds_id} only supports the following topic variants {qcls._fields[1:]}"
+        df = pd.DataFrame(ds.queries_iter())
+
+        df.rename(columns={"query_id": "qid"}, inplace=True) # pyterrier uses "qid"
+
+        if variant is not None:
+            df.rename(columns={variant: "query"}, inplace=True) # user specified which version of the query they want
+            df.drop(df.columns.difference(['qid','query']), 1, inplace=True)
+        elif len(qcls._fields) == 2:
+            # auto-rename single query field to "query" if there's only query_id and that field
+            df.rename(columns={qcls._fields[1]: "query"}, inplace=True)
+        else:
+            print(f'There are multiple query fields available: {qcls._fields[1:]}. To use with pyterrier, provide variant or modify dataframe to add query column.')
+
+        return df
+
+    def get_topics_lang(self):
+        ds = self.irds_ref()
+        if ds.has_queries():
+            return ds.queries_lang()
+        return None
+
+    def get_qrels(self, variant=None):
+        """ 
+            Returns the qrels, as a dataframe, ready for evaluation.
+        """
+        ds = self.irds_ref()
+        assert ds.has_qrels(), f"{self._irds_id} doesn't support get_qrels"
+        qrelcls = ds.qrels_cls()
+        qrel_fields = [f for f in qrelcls._fields if f not in ('query_id', 'doc_id', 'iteration')]
+        assert variant is None or variant in qrel_fields, f"{self._irds_id} only supports the following qrel variants {qrel_fields}"
+        df = pd.DataFrame(ds.qrels_iter())
+
+        # pyterrier uses "qid" and "docno"
+        df.rename(columns={
+            "query_id": "qid",
+            "doc_id": "docno"}, inplace=True)
+
+        # pyterrier uses "label"
+        if variant is not None:
+            df.rename(columns={variant: "label"}, inplace=True)
+        if len(qrel_fields) == 1:
+            # usually "relevance"
+            df.rename(columns={qrel_fields[0]: "label"}, inplace=True)
+        elif 'relevance' in qrel_fields:
+            print(f'There are multiple qrel fields available: {qrel_fields}. Defaulting to "relevance", but to use a different one, supply variant')
+            df.rename(columns={'relevance': "label"}, inplace=True)
+        else:
+            print(f'There are multiple qrel fields available: {qrel_fields}. To use with pyterrier, provide variant or modify dataframe to add query column.')
+
+        return df
+
+    def _describe_component(self, component):
+        ds = self.irds_ref()
+        if component == "topics":
+            if ds.has_queries():
+                fields = ds.queries_cls()._fields[1:]
+                if len(fields) > 1:
+                    return list(fields)
+                return True
+            return None
+        if component == "qrels":
+            if ds.has_qrels():
+                fields = [f for f in ds.qrels_cls()._fields if f not in ('query_id', 'doc_id', 'iteration')]
+                if len(fields) > 1:
+                    return list(fields)
+                return True
+            return None
+        if component == "corpus":
+            return ds.has_docs() or None
+        return None
+
+    def info_url(self):
+        top_id = self._irds_id.split('/', 1)[0]
+        suffix = f'#{self._irds_id}' if top_id != self._irds_id else ''
+        return f'https://ir-datasets.com/{top_id}.html{suffix}'
+
+    def __repr__(self):
+        return f"IRDSDataset({repr(self._irds_id)})"
+
+
+def passage_generate(dataset):
+    for filename in dataset.get_corpus():
+        with autoopen(filename, 'rt') as corpusfile:
+            for l in corpusfile: #for each line
+                docno, passage = l.split("\t")
+                yield {'docno' : docno, 'text' : passage}
+
 
 ANTIQUE_FILES = {
     "topics" : {
@@ -225,6 +402,8 @@ ANTIQUE_FILES = {
     },
     "corpus" : 
         [("antique-collection.txt", "http://ciir.cs.umass.edu/downloads/Antique/antique-collection.txt")],
+    "info_url" : "https://ciir.cs.umass.edu/downloads/Antique/readme.txt",
+    "corpus_iter" : passage_generate
 }
 
 TREC_COVID_FILES = {
@@ -253,8 +432,16 @@ TREC_COVID_FILES = {
         "docids-rnd4" : ("docids-rnd4.txt", "https://ir.nist.gov/covidSubmit/data/docids-rnd4.txt"),
         "docids-rnd5" : ("docids-rnd5.txt", "https://ir.nist.gov/covidSubmit/data/docids-rnd5.txt")
     },
+    "info_url"  : "https://ir.nist.gov/covidSubmit/"
 }
 
+def msmarco_document_generate(dataset):
+    for filename in dataset.get_corpus(variant="corpus-tsv"):
+        with autoopen(filename, 'rt') as corpusfile:
+            for l in corpusfile: #for each line
+                docno, url, title, passage = l.split("\t")
+                yield {'docno' : docno, 'url' : url, 'title' : title, 'text' : passage}
+
 TREC_DEEPLEARNING_DOCS_MSMARCO_FILES = {
     "corpus" : 
         [("msmarco-docs.trec.gz", "https://msmarco.blob.core.windows.net/msmarcoranking/msmarco-docs.trec.gz")],
@@ -273,7 +460,9 @@ TREC_DEEPLEARNING_DOCS_MSMARCO_FILES = {
             "train" : ("msmarco-doctrain-qrels.tsv.gz", "https://msmarco.blob.core.windows.net/msmarcoranking/msmarco-doctrain-qrels.tsv.gz"),
             "dev" : ("msmarco-docdev-qrels.tsv.gz", "https://msmarco.blob.core.windows.net/msmarcoranking/msmarco-docdev-qrels.tsv.gz"),
             "test" : ("2019qrels-docs.txt", "https://trec.nist.gov/data/deep/2019qrels-docs.txt")
-        }
+        },
+    "info_url" : "https://microsoft.github.io/msmarco/",
+    "corpus_iter" : msmarco_document_generate
 }
 
 TREC_DEEPLEARNING_PASSAGE_MSMARCO_FILES = {
@@ -295,7 +484,9 @@ TREC_DEEPLEARNING_PASSAGE_MSMARCO_FILES = {
             "train" : ("qrels.train.tsv", "https://msmarco.blob.core.windows.net/msmarcoranking/qrels.train.tsv"),
             "dev" : ("qrels.dev.tsv", "https://msmarco.blob.core.windows.net/msmarcoranking/qrels.dev.tsv"),
             "test-2019" : ("2019qrels-docs.txt", "https://trec.nist.gov/data/deep/2019qrels-pass.txt")
-        }
+        },
+    "info_url" : "https://microsoft.github.io/MSMARCO-Passage-Ranking/",
+    "corpus_iter" : passage_generate
 }
 
 # remove WT- prefix from topics
@@ -310,12 +501,13 @@ def remove_prefix(self, component, variant):
     return (topics, "direct")
 
 
-# a function to fix the namedpage TREC Web track 2002
+# a function to fix the namedpage TREC Web tracks 2001 and 2002
 def parse_desc_only(self, component, variant):
     import pyterrier as pt
-    file, type = self._get_one_file("topics_special_np")
+    file, type = self._get_one_file("topics_desc_only", variant=variant)
     topics = pt.io.read_topics(file, format="trec", whitelist=["DESC"], blacklist=None)
     topics["qid"] = topics.apply(lambda row: row["qid"].replace("NP", ""), axis=1)
+    topics["qid"] = topics.apply(lambda row: row["qid"].replace("EP", ""), axis=1)
     return (topics, "direct")
 
 TREC_WT_2002_FILES = {
@@ -324,14 +516,15 @@ TREC_WT_2002_FILES = {
             "td" : ("webtopics_551-600.txt.gz", "https://trec.nist.gov/data/topics_eng/webtopics_551-600.txt.gz", "trec"),
             "np" : parse_desc_only
         },
-    "topics_special_np" : [
-        ("webnamed_page_topics.1-150.txt.gz", "https://trec.nist.gov/data/topics_eng/webnamed_page_topics.1-150.txt.gz", "trec")
-    ],
+    "topics_desc_only" : {
+        "np" : ("webnamed_page_topics.1-150.txt.gz", "https://trec.nist.gov/data/topics_eng/webnamed_page_topics.1-150.txt.gz", "trec")
+    },
     "qrels" : 
         { 
             "np" : ("qrels.named-page.txt.gz", "https://trec.nist.gov/data/qrels_eng/qrels.named-page.txt.gz"),
             "td" : ("qrels.distillation.txt.gz", "https://trec.nist.gov/data/qrels_eng/qrels.distillation.txt.gz")
-        }
+        },
+    "info_url" : "https://trec.nist.gov/data/t11.web.html",
 }
 
 TREC_WT_2003_FILES = {
@@ -344,7 +537,8 @@ TREC_WT_2003_FILES = {
         { 
             "np" : ("qrels.named-page.txt.gz", "https://trec.nist.gov/data/qrels_eng/qrels.named-page.txt.gz"),
             "td" : ("qrels.distillation.2003.txt", "https://trec.nist.gov/data/qrels_eng/qrels.distillation.2003.txt")
-        }
+        },
+    "info_url" : "https://trec.nist.gov/data/t12.web.html",
 }
 
 def filter_on_qid_type(self, component, variant):
@@ -378,13 +572,16 @@ TREC_WT_2004_FILES = {
             "td" : filter_on_qid_type,
             "np" : filter_on_qid_type,
             "all" : ("04.qrels.web.mixed.txt", "https://trec.nist.gov/data/web/04.qrels.web.mixed.txt")
-        }
+        },
+    "info_url" : "https://trec.nist.gov/data/t13.web.html",
 }
 
 FIFTY_PCT_INDEX_BASE = "http://www.dcs.gla.ac.uk/~craigm/IR_HM/"
 FIFTY_PCT_FILES = {
-    "index":
-        [(filename, FIFTY_PCT_INDEX_BASE + "index/" + filename) for filename in ["data.meta-0.fsomapfile"] + STANDARD_TERRIER_INDEX_FILES],
+    "index": {
+        "ex1" : [(filename, FIFTY_PCT_INDEX_BASE + "index/" + filename) for filename in ["data.meta-0.fsomapfile"] + STANDARD_TERRIER_INDEX_FILES],
+        "ex2" : [(filename, FIFTY_PCT_INDEX_BASE + "index_block_fields_2021_content/" + filename) for filename in ["data.meta-0.fsomapfile", "data-pagerank.oos"] + STANDARD_TERRIER_INDEX_FILES],   
+    },
     "topics": { 
             "training" : ("training.topics", FIFTY_PCT_INDEX_BASE + "topics/" + "training.topics", "trec"),
             "validation" : ("validation.topics", FIFTY_PCT_INDEX_BASE + "topics/" + "validation.topics", "trec"),
@@ -422,7 +619,8 @@ TREC_WT_2009_FILES = {
         "adhoc" : ("prels.1-50.gz", "https://trec.nist.gov/data/web/09/prels.1-50.gz"),
         "adhoc.catA" : ("prels.catA.1-50.gz", "https://trec.nist.gov/data/web/09/prels.catA.1-50.gz"),
         "adhoc.catB" : ("prels.catB.1-50.gz", "https://trec.nist.gov/data/web/09/prels.catB.1-50.gz")
-    }
+    },
+    "info_url" : "https://trec.nist.gov/data/web09.html",
 }
 
 TREC_WT_2010_FILES = {
@@ -432,7 +630,8 @@ TREC_WT_2010_FILES = {
     "qrels" : 
         { 
             "adhoc" : ("qrels.adhoc", "https://trec.nist.gov/data/web/10/10.adhoc-qrels.final")
-        }
+        },
+    "info_url" : "https://trec.nist.gov/data/web10.html",
 }
 
 TREC_WT_2011_FILES = {
@@ -442,7 +641,8 @@ TREC_WT_2011_FILES = {
     "qrels" : 
         { 
             "adhoc" : ("qrels.adhoc", "https://trec.nist.gov/data/web/11/qrels.adhoc")
-        }
+        },
+    "info_url" : "https://trec.nist.gov/data/web2011.html",
 }
 
 TREC_WT_2012_FILES = {
@@ -452,12 +652,31 @@ TREC_WT_2012_FILES = {
     "qrels" : 
         { 
             "adhoc" : ("qrels.adhoc", "https://trec.nist.gov/data/web/12/qrels.adhoc")
-        }
+        },
+    "info_url" : "https://trec.nist.gov/data/web2012.html",
 }
 
 TREC_WT2G_FILES = {
     "qrels" : [ ("qrels.trec8.small_web.gz", "https://trec.nist.gov/data/qrels_eng/qrels.trec8.small_web.gz") ],
-    "topics" : [ (  "topics.401-450.gz", "https://trec.nist.gov/data/topics_eng/topics.401-450.gz" ) ]
+    "topics" : [ (  "topics.401-450.gz", "https://trec.nist.gov/data/topics_eng/topics.401-450.gz" ) ],
+    "info_url" : "https://trec.nist.gov/data/t8.web.html",
+}
+
+TREC_WT10G_FILES = {
+    "qrels" : {
+        "trec9" : ("qrels.trec9.main_web.gz", "https://trec.nist.gov/data/qrels_eng/qrels.trec9.main_web.gz"),
+        "trec10-adhoc" : ("qrels.trec10.main_web.gz", "https://trec.nist.gov/data/qrels_eng/qrels.trec10.main_web.gz"),
+        "trec10-hp" : ("qrels.trec10.entrypage.gz", "https://trec.nist.gov/data/qrels_eng/qrels.trec10.entrypage.gz"),
+    },
+    "topics" : {
+        "trec9" : (  "topics.451-500.gz", "https://trec.nist.gov/data/topics_eng/topics.451-500.gz" ),
+        "trec10-adhoc" : (  "topics.501-550.txt", "https://trec.nist.gov/data/topics_eng/topics.501-550.txt" ),
+         "trec10-hp" : parse_desc_only
+    },
+    "topics_desc_only" : {
+         "trec10-hp" : (  "entry_page_topics.1-145.txt", "https://trec.nist.gov/data/topics_eng/entry_page_topics.1-145.txt" ),
+    },
+    "info_url" : "https://trec.nist.gov/data/t9.web.html",
 }
 
 def _merge_years(self, component, variant):
@@ -491,16 +710,19 @@ TREC_TB_FILES = {
 
         "2005-np" : ( "05.np_qrels", "https://trec.nist.gov/data/terabyte/05/05.np_qrels"),
         "2006-np" : ( "qrels.tb06.np", "https://trec.nist.gov/data/terabyte/06/qrels.tb06.np"),
-    }
+    },
+    "info_url" : "https://trec.nist.gov/data/terabyte.html"
 }
 
 TREC_ROBUST_04_FILES = {
     "qrels" : [ ("qrels.robust2004.txt", "https://trec.nist.gov/data/robust/qrels.robust2004.txt") ],
-    "topics" : [ (  "04.testset.gz", "https://trec.nist.gov/data/robust/04.testset.gz" ) ]
+    "topics" : [ (  "04.testset.gz", "https://trec.nist.gov/data/robust/04.testset.gz" ) ],
+    "info_url" : "https://trec.nist.gov/data/t13_robust.html",
 }
 TREC_ROBUST_05_FILES = {
     "qrels" : [ ("TREC2005.qrels.txt", "https://trec.nist.gov/data/robust/05/TREC2005.qrels.txt") ],
-    "topics" : [ (  "05.50.topics.txt", "https://trec.nist.gov/data/robust/05/05.50.topics.txt" ) ]
+    "topics" : [ (  "05.50.topics.txt", "https://trec.nist.gov/data/robust/05/05.50.topics.txt" ) ],
+    "info_url" : "https://trec.nist.gov/data/t14_robust.html",
 }
 
 TREC_PRECISION_MEDICINE_FILES = {
@@ -522,9 +744,12 @@ TREC_PRECISION_MEDICINE_FILES = {
         "qrels-2019-trials" : ("qrels-2019-trials.txt", "https://trec.nist.gov/data/precmed/qrels-treceval-trials.38.txt"),
         "qrels-2019-abstracts-sample" : ("qrels-2019-abstracts-sample.txt", "https://trec.nist.gov/data/precmed/qrels-sampleval-abstracts.2019.txt"),
         "qrels-2019-trials-sample" : ("qrels-2019-trials-sample.txt", "https://trec.nist.gov/data/precmed/qrels-sampleval-trials.38.txt")
-    }
+    },
+    "info_url" : "https://trec.nist.gov/data/precmed.html",
 }
 
+
+
 VASWANI_CORPUS_BASE = "https://raw.githubusercontent.com/terrier-org/pyterrier/master/tests/fixtures/vaswani_npl/"
 VASWANI_INDEX_BASE = "https://raw.githubusercontent.com/terrier-org/pyterrier/master/tests/fixtures/index/"
 VASWANI_FILES = {
@@ -535,13 +760,14 @@ VASWANI_FILES = {
     "qrels":
         [("qrels", VASWANI_CORPUS_BASE + "qrels")],
     "index":
-        [(filename, VASWANI_INDEX_BASE + filename) for filename in STANDARD_TERRIER_INDEX_FILES + ["data.meta-0.fsomapfile"]]
+        [(filename, VASWANI_INDEX_BASE + filename) for filename in STANDARD_TERRIER_INDEX_FILES + ["data.meta-0.fsomapfile"]],
+    "info_url" : "http://ir.dcs.gla.ac.uk/resources/test_collections/npl/",
 }
 
 DATASET_MAP = {
     # used for UGlasgow teaching
     "50pct" : RemoteDataset("50pct", FIFTY_PCT_FILES),
-    # umass antique corpus - see http://ciir.cs.umass.edu/downloads/Antique/
+    # umass antique corpus - see http://ciir.cs.umass.edu/downloads/Antique/ 
     "antique" : RemoteDataset("antique", ANTIQUE_FILES),
     # generated from http://ir.dcs.gla.ac.uk/resources/test_collections/npl/
     "vaswani": RemoteDataset("vaswani", VASWANI_FILES),
@@ -555,6 +781,8 @@ DATASET_MAP = {
     "trec-covid" : RemoteDataset("trec-covid", TREC_COVID_FILES),
     #wt2g
     "trec-wt2g" : RemoteDataset("trec-wt2g", TREC_WT2G_FILES),
+    #wt10g
+    "trec-wt10g" : RemoteDataset("trec-wt10g", TREC_WT10G_FILES),
     #.gov
     "trec-wt-2002" : RemoteDataset("trec-wt-2002", TREC_WT_2002_FILES),
     "trec-wt-2003" : RemoteDataset("trec-wt-2003", TREC_WT_2002_FILES),
@@ -566,6 +794,19 @@ DATASET_MAP = {
     "trec-wt-2012" : RemoteDataset("trec-wt-2012", TREC_WT_2012_FILES),
 }
 
+
+# Include all datasets from ir_datasets with "irds:" prefix so they don't conflict with pt dataset names
+# Results in records like:
+# irds:antique
+# irds:antique/test
+# irds:antique/test/non-offensive
+# irds:antique/train
+# ...
+import ir_datasets
+for ds_id in ir_datasets.registry:
+    DATASET_MAP[f'irds:{ds_id}'] = IRDSDataset(ds_id)
+
+
 def get_dataset(name, **kwargs):
     """
         Get a dataset by name
@@ -580,9 +821,10 @@ def datasets():
     """
     return DATASET_MAP.keys()
 
-def list_datasets():
+def list_datasets(en_only=True):
     """
-        Returns a dataframe of all datasets, listing which topics, qrels, corpus files or indices are available
+        Returns a dataframe of all datasets, listing which topics, qrels, corpus files or indices are available.
+        By default, filters to only datasets with both a corpus and topics in English.
     """
     import pandas as pd
     rows=[]
@@ -591,7 +833,15 @@ def list_datasets():
         rows.append([
             k, 
             dataset._describe_component("topics"), 
+            dataset.get_topics_lang(), 
             dataset._describe_component("qrels"), 
             dataset._describe_component("corpus"), 
-            dataset._describe_component("index") ])
-    return pd.DataFrame(rows, columns=["dataset", "topics", "qrels", "corpus", "index"])
\ No newline at end of file
+            dataset.get_corpus_lang(), 
+            dataset._describe_component("index"), 
+            dataset.info_url() ])
+    result = pd.DataFrame(rows, columns=["dataset", "topics", "topics_lang", "qrels", "corpus", "corpus_lang", "index", "info_url"])
+    if en_only:
+        topics_filter = (result['topics'].isnull()) | (result['topics_lang'] == 'en')
+        corpus_filter = (result['corpus'].isnull()) | (result['corpus_lang'] == 'en')
+        result = result[topics_filter & corpus_filter]
+    return result
