@@ -10,6 +10,14 @@ import pandas as pd
 import os
 import enum
 import json
+import tempfile
+import contextlib
+import threading
+import select
+import math
+from warnings import warn
+from collections import deque
+from typing import List, Dict
 
 StringReader = None
 HashMap = None
@@ -31,6 +39,8 @@ Properties = None
 CLITool = None
 IndexRef = None
 IndexFactory = None
+StructureMerger = None
+BlockStructureMerger = None
 
 
 # lastdoc ensures that a Document instance from a Collection is not GCd before Java has used it.
@@ -57,6 +67,8 @@ def run_autoclass():
     global CLITool
     global IndexRef
     global IndexFactory
+    global StructureMerger
+    global BlockStructureMerger
 
     StringReader = autoclass("java.io.StringReader")
     HashMap = autoclass("java.util.HashMap")
@@ -78,7 +90,20 @@ def run_autoclass():
     CLITool = autoclass("org.terrier.applications.CLITool")
     IndexRef = autoclass('org.terrier.querying.IndexRef')
     IndexFactory = autoclass('org.terrier.structures.IndexFactory')
+    StructureMerger = autoclass("org.terrier.structures.merging.StructureMerger")
+    BlockStructureMerger = autoclass("org.terrier.structures.merging.BlockStructureMerger")
 
+def createAsList(files_path):
+    """
+    Helper method to be used by child indexers to add files to Java List
+    Returns:
+        Created Java List
+    """
+    if isinstance(files_path, str):
+        asList = Arrays.asList(files_path)
+    elif isinstance(files_path, list):
+        asList = Arrays.asList(*files_path)
+    return asList
 
 # Using enum class create enumerations
 class IndexingType(enum.Enum):
@@ -89,6 +114,7 @@ class IndexingType(enum.Enum):
     SINGLEPASS = 2 #: A single-pass indexing regime, which builds an inverted index directly. No direct index structure is created. Typically is faster than classical indexing.
     MEMORY = 3 #: An in-memory index. No direct index is created.
 
+
 class Indexer:
     """
     Parent class. It can be used to load an existing index.
@@ -172,45 +198,57 @@ class Indexer:
 
     def createIndexer(self):
         """
-        Check `single_pass` and
+        TODO
+        Checks `self.type` and
         - if false, check `blocks` and create a BlockIndexer if true, else create BasicIndexer
         - if true, check `blocks` and create a BlockSinglePassIndexer if true, else create BasicSinglePassIndexer
         Returns:
             Created index object
         """
+        
+        Indexer, _ = self.indexerAndMergerClasses()
+        if Indexer is BasicMemoryIndexer:
+            index = Indexer()
+        else:
+            index = Indexer(self.index_dir, "data")
+        assert index is not None
+        return index
+
+    def indexerAndMergerClasses(self):
+        """
+        Check `single_pass` and
+        - if false, check `blocks` and create a BlockIndexer if true, else create BasicIndexer
+        - if true, check `blocks` and create a BlockSinglePassIndexer if true, else create BasicSinglePassIndexer
+        Returns:
+            type objects for indexer and merger for the given configuration
+        """
+        self.properties['indexer.meta.forward.keys'] = ','.join(self.meta.keys())
+        self.properties['indexer.meta.forward.keylens'] = ','.join([str(l) for l in self.meta.values()])
         ApplicationSetup.getProperties().putAll(self.properties)
-        # ApplicationSetup.bootstrapInitialisation(self.properties)
         if self.type is IndexingType.SINGLEPASS:
             if self.blocks:
-                index = BlockSinglePassIndexer(self.index_dir, "data")
+                Indexer = BlockSinglePassIndexer
+                Merger = BlockStructureMerger
             else:
-                index = BasicSinglePassIndexer(self.index_dir, "data")
+                Indexer = BasicSinglePassIndexer
+                Merger = StructureMerger
         elif self.type is IndexingType.CLASSIC:
             if self.blocks:
-                index = BlockIndexer(self.index_dir, "data")
+                Indexer = BlockIndexer
+                Merger = BlockStructureMerger
             else:
-                index = BasicIndexer(self.index_dir, "data")
+                Indexer = BasicIndexer
+                Merger = StructureMerger
         elif self.type is IndexingType.MEMORY:
             if self.blocks:
                 raise Exception("Memory indexing with positions not yet implemented")
             else:
-                index = BasicMemoryIndexer()
+                Indexer = BasicMemoryIndexer
+                Merger = None
         else:
             raise Exception("Unknown indexer type")
-        assert index is not None
-        return index
-
-    def createAsList(self, files_path):
-        """
-        Helper method to be used by child indexers to add files to Java List
-        Returns:
-            Created Java List
-        """
-        if isinstance(files_path, str):
-            asList = Arrays.asList(files_path)
-        elif isinstance(files_path, list):
-            asList = Arrays.asList(*files_path)
-        return asList
+        assert Indexer is not None
+        return Indexer, Merger
 
     def getIndexStats(self):
         """
@@ -323,26 +361,9 @@ class DFIndexer(Indexer):
         # we need to prevent collectionIterator from being GCd, so assign to a variable that outlives the indexer
         collectionIterator, meta_lengths = DFIndexUtils.create_javaDocIterator(text, *args, **kwargs)
         
-        # generate the metadata properties, set their lengths automatically
-        mprop1=""
-        mprop2=""
-        mprop1_def=None
-        mprop2_def=None
-
-        # keep track of the previous settings of these indexing properties
-        default_props = ApplicationSetup.getProperties()
-        if default_props.containsKey("indexer.meta.forward.keys"):
-            mprop1_def = default_props.get("indexer.meta.forward.keys")
-        if default_props.containsKey("indexer.meta.forward.keylens"):
-            mprop2_def = default_props.get("indexer.meta.forward.keylens")
-
-        # update the indexing properties
-        for k in meta_lengths:
-            mprop1 += k+ ","
-            mprop2 += str(meta_lengths[k]) + ","
-        ApplicationSetup.setProperty("indexer.meta.forward.keys", mprop1[:-1])
-        ApplicationSetup.setProperty("indexer.meta.forward.keylens", mprop2[:-1])
-
+        # record the metadata key names and the length of the values
+        self.meta = meta_lengths
+        
         # make a Collection class for Terrier
         javaDocCollection = autoclass("org.terrier.python.CollectionFromDocumentIterator")(collectionIterator)
         if self.verbose:
@@ -355,16 +376,6 @@ class DFIndexer(Indexer):
         self.index_called = True
         collectionIterator = None
 
-        # this block is for restoring the indexing config
-        if mprop1_def is not None:
-            ApplicationSetup.setProperty("indexer.meta.forward.keys", mprop1_def)
-        else:
-            default_props.remove("indexer.meta.forward.keys")
-        if mprop2_def is not None:
-            ApplicationSetup.setProperty("indexer.meta.forward.keylens", mprop2_def)
-        else:
-            default_props.remove("indexer.meta.forward.keylens")
-
         if self.type is IndexingType.MEMORY:
             return index.getIndex().getIndexRef()
         return IndexRef.of(self.index_dir + "/data.properties")
@@ -444,12 +455,44 @@ class FlatJSONDocumentIterator(PythonJavaClass):
             return lastdoc
         return None
 
-class IterDictIndexer(Indexer):
-    """
-    Use this Indexer if you wish to index an iter of dicts (possibly with multiple fields)
 
+class _BaseIterDictIndexer(Indexer):
+    def __init__(self, index_path, *args, threads=1, **kwargs):
+        super().__init__(index_path, *args, **kwargs)
+        self.threads = threads
+
+    def _setup(self, fields, meta, meta_lengths):
+        """
+        Index the specified iter of dicts with the (optional) specified fields
+
+        Args:
+            it(iter[dict]): an iter of document dict to be indexed
+            fields(list[str]): keys to be indexed as fields
+            meta(dict[str,int]): keys to be considered as metdata, and their lengths
+            meta_lengths(list[int]): deprecated
+        """
+        self.checkIndexExists()
+        if isinstance(meta, dict):
+            self.meta = meta
+        else: 
+            if meta_lengths is None:
+                # the ramifications of setting all lengths to a large value is an overhead in memory usage during decompression
+                meta_lengths = ['512'] * len(meta)
+            self.meta = { k:v for k,v in zip( meta, meta_lengths)}
+
+        self.setProperties(**{
+            'metaindex.compressed.crop.long' : 'true',
+            'FieldTags.process': ','.join(fields),
+            'FieldTags.casesensitive': 'true',
+        })
+
+
+class _IterDictIndexer_nofifo(_BaseIterDictIndexer):
     """
-    def index(self, it, fields=('text',), meta=('docno',), meta_lengths=None):
+    Use this Indexer if you wish to index an iter of dicts (possibly with multiple fields).
+    This version is used for Windows -- which doesn't support the faster fifo implementation.
+    """
+    def index(self, it, fields=('text',), meta=('docno',), meta_lengths=None, threads=None):
         """
         Index the specified iter of dicts with the (optional) specified fields
 
@@ -459,16 +502,8 @@ class IterDictIndexer(Indexer):
             meta(list[str]): keys to be considered as metdata
             meta_lengths(list[int]): length of metadata, defaults to 512 characters
         """
-        self.checkIndexExists()
-        # What are the ramifications of setting all lengths to a large value like this? (storage cost?)
-        if meta_lengths is None:
-            meta_lengths = ['512'] * len(meta)
-        self.setProperties(**{
-            'FieldTags.process': ','.join(fields),
-            'FieldTags.casesensitive': 'true',
-            'indexer.meta.forward.keys': ','.join(meta),
-            'indexer.meta.forward.keylens': ','.join([str(l) for l in meta_lengths])
-        })
+        self._setup(fields, meta, meta_lengths)
+        assert self.threads == 1, 'IterDictIndexer does not support multiple threads on Windows'
         # we need to prevent collectionIterator from being GCd
         collectionIterator = FlatJSONDocumentIterator(iter(it)) # force it to be iter
         javaDocCollection = autoclass("org.terrier.python.CollectionFromDocumentIterator")(collectionIterator)
@@ -482,6 +517,95 @@ class IterDictIndexer(Indexer):
             return index.getIndex().getIndexRef()
         return IndexRef.of(self.index_dir + "/data.properties")
 
+
+class _IterDictIndexer_fifo(_BaseIterDictIndexer):
+    """
+    Use this Indexer if you wish to index an iter of dicts (possibly with multiple fields).
+    This version is optimized by using multiple threads and POSIX fifos to tranfer data,
+    which ends up being much faster.
+    """
+    def index(self, it, fields=('text',), meta=('docno',), meta_lengths=None):
+        """
+        Index the specified iter of dicts with the (optional) specified fields
+
+        Args:
+            it(iter[dict]): an iter of document dict to be indexed
+            fields(list[str]): keys to be indexed as fields
+            meta(list[str]): keys to be considered as metdata
+            meta_lengths(list[int]): length of metadata, defaults to 512 characters
+        """
+        CollectionFromDocumentIterator = autoclass("org.terrier.python.CollectionFromDocumentIterator")
+        JsonlDocumentIterator = autoclass("org.terrier.python.JsonlDocumentIterator")
+        ParallelIndexer = autoclass("org.terrier.python.ParallelIndexer")
+
+        self._setup(fields, meta, meta_lengths)
+
+        os.makedirs(self.index_dir, exist_ok=True) # ParallelIndexer expects the directory to exist
+
+        Indexer, Merger = self.indexerAndMergerClasses()
+
+        assert self.threads > 0, "threads must be positive"
+        if Indexer is BasicMemoryIndexer:
+            assert self.threads == 1, 'IterDictIndexer does not support multiple threads for IndexingType.MEMORY'
+        if self.threads > 1:
+            warn('Using multiple threads results in a non-deterministic ordering of document in the index. For deterministic behavior, use threads=1')
+
+        # Document iterator
+        fifos = []
+        j_collections = []
+        with tempfile.TemporaryDirectory() as d:
+            # Make a POSIX FIFO with associated java collection for each thread to use
+            for i in range(self.threads):
+                fifo = f'{d}/docs-{i}.jsonl'
+                os.mkfifo(fifo)
+                j_collections.append(CollectionFromDocumentIterator(JsonlDocumentIterator(fifo)))
+                fifos.append(fifo)
+
+            # Start dishing out the docs to the fifos
+            threading.Thread(target=self._write_fifos, args=(it, fifos), daemon=True).start()
+
+            # Different process for memory indexer (still taking advantage of faster fifos)
+            if Indexer is BasicMemoryIndexer:
+                index = Indexer()
+                index.index(j_collections)
+                return index.getIndex().getIndexRef()
+
+            # Start the indexing threads
+            ParallelIndexer.buildParallel(j_collections, self.index_dir, Indexer, Merger)
+            return IndexRef.of(self.index_dir + "/data.properties")
+
+    def _write_fifos(self, it, fifos):
+        c = len(fifos)
+        with contextlib.ExitStack() as stack:
+            fifos = [stack.enter_context(open(f, 'wt')) for f in fifos]
+            ready = None
+            for doc in it:
+                if not ready: # either first iteration or deque is empty
+                    if len(fifos) > 1:
+                        # Not all the fifos may be ready yet for the next document. Rather than
+                        # witing for the next one to finish up, go ahead and can check wich are ready
+                        # with the select syscall. This will block until at least one is ready. This
+                        # optimization can actually have a pretty big impact-- on CORD19, indexing
+                        # with 8 threads was 30% faster with this.
+                        _, ready, _ = select.select([], fifos, [])
+                        ready = deque(ready)
+                    else:
+                        # single threaded mode
+                        ready = deque(fifos)
+                fifo = ready.popleft()
+                json.dump(doc, fifo)
+                fifo.write('\n')
+
+
+# Windows doesn't support fifos -- so we have 2 versions.
+# Choose which one to expose based on whether os.mkfifo exists.
+if hasattr(os, 'mkfifo'):
+    IterDictIndexer = _IterDictIndexer_fifo
+else:
+    IterDictIndexer = _IterDictIndexer_nofifo
+IterDictIndexer.__name__ = 'IterDictIndexer' # trick sphinx into not using "alias of"
+
+
 class TRECCollectionIndexer(Indexer):
     type_to_class = {
         'trec' : 'org.terrier.indexing.TRECCollection',
@@ -493,7 +617,17 @@ class TRECCollectionIndexer(Indexer):
         Use this Indexer if you wish to index a TREC formatted collection
     """
 
-    def __init__(self, index_path, blocks=False, overwrite=False, type=IndexingType.CLASSIC, collection="trec", verbose=False):
+    def __init__(self, 
+            index_path : str, 
+            blocks : bool = False, 
+            overwrite : bool = False, 
+            type : IndexingType =IndexingType.CLASSIC, 
+            collection : str = "trec", 
+            verbose : bool = False,
+            meta : Dict[str,int] = {"docno" : 20},
+            meta_reverse : List[str] = ["docno"],
+            meta_tags : Dict[str,str] = {}
+            ):
         """
         Init method
 
@@ -510,6 +644,9 @@ class TRECCollectionIndexer(Indexer):
                 collection = TRECCollectionIndexer.type_to_class[collection]
         self.collection = collection.split(",")
         self.verbose = verbose
+        self.meta = meta
+        self.meta_reverse = meta_reverse
+        self.meta_tags = meta_tags
     
 
     def index(self, files_path):
@@ -521,7 +658,20 @@ class TRECCollectionIndexer(Indexer):
         """
         self.checkIndexExists()
         index = self.createIndexer()
-        asList = self.createAsList(files_path)
+        asList = createAsList(files_path)
+
+        abstract_tags=self.meta_tags.values()
+        abstract_names=self.meta_tags.keys()
+        abstract_lengths=[str(self.meta[name]) for name in abstract_names]
+
+        ApplicationSetup.setProperty("TaggedDocument.abstracts", ",".join(abstract_names))
+        # The tags from which to save the text. ELSE is special tag name, which means anything not consumed by other tags.
+        ApplicationSetup.setProperty("TaggedDocument.abstracts.tags", ",".join(abstract_tags))
+        # The max lengths of the abstracts. Abstracts will be cropped to this length. Defaults to empty.
+        ApplicationSetup.setProperty("TaggedDocument.abstracts.lengths", ",".join(abstract_lengths))
+        # Should the tags from which we create abstracts be case-sensitive
+        ApplicationSetup.setProperty("TaggedDocument.abstracts.tags.casesensitive", "false")
+
         cls_string = autoclass("java.lang.String")._class
         cls_list = autoclass("java.util.List")._class
         colObj = autoclass("org.terrier.indexing.CollectionFactory").loadCollections(
@@ -549,8 +699,8 @@ class FilesIndexer(Indexer):
 
     def __init__(self, index_path, *args, **kwargs):
         super().__init__(index_path, args, kwargs)
-        self.properties["indexer.meta.forward.keys"]="docno,filename"
-        self.properties["indexer.meta.forward.keylens"]="20,512"
+        self.meta=["docno", "filename"] 
+        self.meta_lengths=[20,512]
 
     def index(self, files_path):
         """
@@ -561,7 +711,7 @@ class FilesIndexer(Indexer):
         """
         self.checkIndexExists()
         index = self.createIndexer()
-        asList = self.createAsList(files_path)
+        asList = createAsList(files_path)
         simpleColl = SimpleFileCollection(asList, False)
         index.index([simpleColl])
         global lastdoc
